@@ -35,7 +35,7 @@ from metview import plotting
 from metview.metviewpy.ipython import is_ipython_active, import_widgets
 from metview.metviewpy import utils
 
-__version__ = "1.14.0"
+__version__ = "1.15.0"
 
 
 # logging.basicConfig(level=logging.DEBUG, format="%(levelname)s - %(message)s")
@@ -760,7 +760,11 @@ class Fieldset(FileBackedValueWithOperators, ContainerValue):
         # e.g. [{level=1000,shortName="t",date=20220101, time=6}, ...]
         fieldsets_to_apply_function_to = [self.select(**x) for x in perms]
         result = Fieldset(
-            fields=[func_to_run(x, **kwargs) for x in fieldsets_to_apply_function_to]
+            fields=[
+                func_to_run(x, **kwargs)
+                for x in fieldsets_to_apply_function_to
+                if len(x) != 0
+            ]
         )
         return result
 
@@ -1334,6 +1338,7 @@ def bind_functions(namespace, module_name=None):
     namespace["map_style_gallery"] = map_style_gallery
     namespace["map_area_gallery"] = map_area_gallery
     namespace["make_geoview"] = make_geoview
+    namespace["arguments"] = met_arguments
     namespace["Fieldset"] = Fieldset
     namespace["Request"] = Request
 
@@ -1643,3 +1648,17 @@ def setoutput(*args, **kwargs):
     else:
         plot.plot_to_jupyter = False
         met_setoutput(*args)
+
+
+def met_arguments():
+    """Emulate the Macro arguments() function"""
+    import sys
+
+    args = sys.argv[1:]
+    # these will all come in as strings; but Macro does a little processing on them
+    # in order to intelligently decide whether their types or string or number;
+    # creating a Request with these values will simulate this behaviour
+    args_dict = {i: i for i in args}
+    modified_args_dict = Request(args_dict).to_dict()
+    modified_args = [modified_args_dict[j] for j in modified_args_dict]
+    return modified_args
