@@ -11,7 +11,6 @@
 
 import copy
 import datetime
-from distutils.version import StrictVersion
 import logging
 import os
 from pathlib import Path
@@ -31,17 +30,19 @@ NEWER = True
 class GribIndexer:
     VECTOR_PARAMS = {
         "wind10m": ["10u", "10v"],
+        "wind100m": ["100u", "100v"],
+        "wind200m": ["200u", "200v"],
         "wind": ["u", "v"],
         "wind3d": ["u", "v", "w"],
     }
 
-    # 0: ecCodes type, 1: pandas type, 2: use in duplicate check
+    # tuple-> 0: ecCodes type, 1: pandas type, 2: Python type 3: use in duplicate check
     DEFAULT_KEYS = {
         "shortName": ("s", str, str, False),
         "paramId": ("l", "Int32", int, False),
         "date": ("l", "Int64", int, True),
         "time": ("l", "Int64", int, True),
-        "step": ("s", str, str, True),
+        "step": ("l", "Int32", int, True),
         "level": ("l", "Int32", int, True),
         "typeOfLevel": ("s", str, str, False),
         "number": ("s", str, str, True),
@@ -54,6 +55,14 @@ class GribIndexer:
     DEFAULT_ECC_KEYS = [f"{k}:{v[0]}" for k, v in DEFAULT_KEYS.items()]
     BLOCK_KEYS = ["shortName", "typeOfLevel"]
 
+    DEFAULT_SORT_KEYS = [
+        "date",
+        "time",
+        "step",
+        "number",
+        "level",
+        "paramId",
+    ]
     DATE_KEYS = {
         k: ("l", "Int64", int)
         for k in ["date", "dataDate", "validityDate", "mars.date", "marsDate"]
@@ -69,6 +78,15 @@ class GribIndexer:
         "_validityDateTime": ("validityDate", "validityTime"),
     }
 
+    KEYS_TO_REPLACE = {
+        ("type", "mars.type"): "marsType",
+        ("stream", "mars.stream"): "marsStream",
+        ("class", "mars.class", "class_"): "marsClass",
+        ("perturbationNumber"): "number",
+        ("mars.date", "marsDate"): "date",
+        ("mars.time", "marsTime"): "time",
+    }
+
     PREDEF_KEYS = copy.deepcopy(DEFAULT_KEYS)
     PREDEF_KEYS.update(DATE_KEYS)
     PREDEF_KEYS.update(TIME_KEYS)
@@ -239,7 +257,7 @@ class GribIndexer:
                     i += 1
         return r
 
-    def _make_dataframe(self, data, columns=None):
+    def _make_dataframe(self, data, sort=False, columns=None):
         if columns is not None:
             df = pd.DataFrame(data, columns=columns)
         else:
@@ -249,30 +267,20 @@ class GribIndexer:
             if self.pd_types.get(c, "") in ["Int32", "Int64"]:
                 df[c].fillna(value=np.nan, inplace=True)
             df = df.astype(self.pd_types)
-        df = GribIndexer._sort_dataframe(df)
+        if sort:
+            df = GribIndexer._sort_dataframe(df)
 
         return df
 
     @staticmethod
-    def _sort_dataframe(df):
-        # the key argument is available from pandas 1.1.0
-        if StrictVersion(pd.__version__) >= StrictVersion("1.1.0"):
-            df = df.sort_values(
-                by=list(df.columns),
-                key=lambda col: col.str.pad(7, side="left", fillchar="0")
-                if col.name == "step"
-                else col,
-            )
-        # for an older pandas version we use this ad hoc method
-        else:
-            step_ori = []
-            if "step" in list(df.columns):
-                step_ori = list(df.step)
-                df.step = df.step.str.pad(7, side="left", fillchar="0")
-            df = df.sort_values(by=list(df.columns))
-            if step_ori:
-                df.step = [step_ori[idx] for idx in df.index]
+    def _sort_dataframe(df, columns=None, ascending=True):
+        if columns is None:
+            columns = list(df.columns)
+        elif not isinstance(columns, list):
+            columns = [columns]
 
+        # mergesoft is a stable sorting algorithm
+        df = df.sort_values(by=columns, ascending=ascending, kind="mergesort")
         df = df.reset_index(drop=True)
         return df
 
@@ -283,7 +291,6 @@ class GribIndexer:
     @staticmethod
     def read_dataframe(key, dir_name):
         # assert len(key) == len(GribIndexer.BLOCK_KEYS)
-        # name = "_".join(key)
         name = key
         f_name = os.path.join(dir_name, f"{name}.csv.gz")
         # LOG.debug("f_name={}".format(f_name))
@@ -308,18 +315,6 @@ class GribIndexer:
     def _convert_query_value(v, col_type):
         # print(f"v={v} {type(v)} {col_type}")
         return v if col_type != "object" else str(v)
-        # if isinstance(v, datetime.datetime):
-        #     t = v.strftime("%Y%m%d%H%M")
-        #     # print(f"  -> {t}")
-        #     return int(t) if col_type != "object" else t
-        # elif isinstance(v, datetime.date):
-        #     t = v.strftime("%Y%m%d")
-        #     return int(t) if col_type != "object" else t
-        # elif isinstance(v, datetime.time):
-        #     t = v.strftime("%H%M")
-        #     return int(t) if col_type != "object" else t
-        # else:
-        #     return v if col_type != "object" else str(v)
 
     @staticmethod
     def _check_datetime_in_filter_input(keys):
@@ -331,11 +326,11 @@ class GribIndexer:
                 keys.get(name_date, []) or keys.get(name_time, [])
             ):
                 raise Exception(
-                    "Cannot specify {name} together with {name_date} and {name_time}!"
+                    f"Cannot specify {name} together with {name_date} and {name_time}!"
                 )
 
     @staticmethod
-    def _convert_filter_value(name, v):
+    def _convert_filter_value(name, val):
         """
         Analyse the filter key-value pairs and perform the necessary conversions
         """
@@ -347,92 +342,91 @@ class GribIndexer:
             valid_name = "_" + valid_name
             name_date = GribIndexer.DATETIME_KEYS[valid_name][0]
             name_time = GribIndexer.DATETIME_KEYS[valid_name][1]
-            for i, t in enumerate(v):
-                v[i] = GribIndexer._to_datetime(name, t)
-                # print(f"t={t} -> {v[i]}")
+            for i, t in enumerate(val):
+                val[i] = GribIndexer._to_datetime(name, t)
+                # print(f"t={t} -> {val[i]}")
             # We add the date and time components with an empty value. So they will be
             # added to the scan, but they will be ignored by the query. Conversely,
             # the datetime key itself will be ignored in the scan, but will be used
             # in the query.
-            return [("_" + name, v), (name_date, []), (name_time, [])]
+            return [("_" + name, val), (name_date, []), (name_time, [])]
         # we convert dates to int
         elif valid_name in GribIndexer.DATE_KEYS:
-            for i, t in enumerate(v):
-                v[i] = int(GribIndexer._to_date(name, t).strftime("%Y%m%d"))
+            for i, t in enumerate(val):
+                d = GribIndexer._to_date(name, t)
+                # for daily climatologies dates where the year is missing the
+                # the a tuple is returned
+                if not isinstance(d, tuple):
+                    val[i] = int(d.strftime("%Y%m%d"))
+                else:
+                    val[i] = d[0] * 100 + d[1]
         # we convert times to int
         elif valid_name in GribIndexer.TIME_KEYS:
-            for i, t in enumerate(v):
-                v[i] = int(GribIndexer._to_time(name, t).strftime("%H%M"))
-                # print(f"t={t} -> {v[i]}")
+            for i, t in enumerate(val):
+                val[i] = int(GribIndexer._to_time(name, t).strftime("%H%M"))
+                # print(f"t={t} -> {val[i]}")
         else:
             pt_type = GribIndexer.PREDEF_PT_TYPES.get(name, None)
             # print(f"name={name} {pt_type}")
             if pt_type is not None:
-                for i, t in enumerate(v):
-                    v[i] = pt_type(t)
-                    # print(f" t={t} -> {v[i]}")
+                for i, t in enumerate(val):
+                    val[i] = pt_type(t)
+                    # print(f" t={t} -> {val[i]}")
 
-        # remap some names to ones already in the default set of indexer keys
-        if name in ["type", "mars.type"]:
-            name = "marsType"
-        elif name in ["stream", "mars.stream"]:
-            name = "marsStream"
-        elif name in ["class", "mars.class", "class_"]:
-            name = "marsClass"
-        elif name in ["perturbationNumber"]:
-            name = "number"
-        elif name in ["mars.date", "marsDate"]:
-            name = "date"
-        elif name in ["mars.time", "marsTime"]:
-            name = "time"
+        # remap some names to the ones already in the default set of indexer keys
+        for k, v in GribIndexer.KEYS_TO_REPLACE.items():
+            if name in k:
+                name = v
 
-        return [(name, v)]
+        return [(name, val)]
 
     @staticmethod
-    def _to_datetime(param, v):
+    def _to_datetime(param, val):
         try:
-            if isinstance(v, datetime.datetime):
-                return v
-            elif isinstance(v, str):
-                return utils.date_from_str(v)
-            elif isinstance(v, (int, float)):
-                return utils.date_from_str(str(v))
+            if isinstance(val, datetime.datetime):
+                return val
+            elif isinstance(val, str):
+                return utils.date_from_str(val)
+            elif isinstance(val, (int, float)):
+                return utils.date_from_str(str(val))
             else:
                 raise
         except:
-            raise Exception(f"Invalid datetime value={v} specified for key={param}")
+            raise Exception(f"Invalid datetime value={val} specified for key={param}")
 
     @staticmethod
-    def _to_date(param, v):
+    def _to_date(param, val):
         try:
-            if isinstance(v, datetime.datetime):
-                return v.date()
-            elif isinstance(v, datetime.date):
-                return v
-            elif isinstance(v, str):
-                return utils.date_from_str(v).date()
-            elif isinstance(v, (int, float)):
-                return utils.date_from_str(str(v)).date()
+            if isinstance(val, datetime.datetime):
+                return val.date()
+            elif isinstance(val, datetime.date):
+                return val
+            elif isinstance(val, str):
+                d = utils.date_from_str(val)
+                return d.date() if not isinstance(d, tuple) else d
+            elif isinstance(val, (int, float)):
+                d = utils.date_from_str(str(val))
+                return d.date() if not isinstance(d, tuple) else d
             else:
                 raise
         except:
-            raise Exception(f"Invalid date value={v} specified for key={param}")
+            raise Exception(f"Invalid date value={val} specified for key={param}")
 
     @staticmethod
-    def _to_time(param, v):
+    def _to_time(param, val):
         try:
-            if isinstance(v, (datetime.datetime)):
-                return v.time()
-            elif isinstance(v, datetime.time):
-                return v
-            elif isinstance(v, str):
-                return utils.time_from_str(v)
-            elif isinstance(v, int):
-                return utils.time_from_str(str(v))
+            if isinstance(val, (datetime.datetime)):
+                return val.time()
+            elif isinstance(val, datetime.time):
+                return val
+            elif isinstance(val, str):
+                return utils.time_from_str(val)
+            elif isinstance(val, int):
+                return utils.time_from_str(str(val))
             else:
                 raise
         except:
-            raise Exception(f"Invalid time value={v} specified for key={param}")
+            raise Exception(f"Invalid time value={val} specified for key={param}")
 
 
 class FieldsetIndexer(GribIndexer):
@@ -443,7 +437,7 @@ class FieldsetIndexer(GribIndexer):
     def scan(self, vector=False):
         data = self._scan(self.db.fs, mapped_params=self.db.mapped_params)
         if data:
-            df = self._make_dataframe(data)
+            df = self._make_dataframe(data, sort=False)
             self.db.blocks["scalar"] = df
             if vector:
                 self._scan_vector()
@@ -452,6 +446,7 @@ class FieldsetIndexer(GribIndexer):
         LOG.info(f" scan fields ...")
         data = {}
         # print(f"fs_len={len(fs)}")
+        # print(f"keys_ecc={self.keys_ecc}")
         if utils.is_fieldset_type(fs) and len(fs) > 0:
             md_vals = fs.grib_get(self.keys_ecc, "key")
             if mapped_params:
@@ -477,7 +472,7 @@ class FieldsetIndexer(GribIndexer):
                     cols = [*self.keys]
                     for i in range(comp_num):
                         cols.extend([f"_msgIndex{i+1}"])
-                    w_df = self._make_dataframe(r, columns=cols)
+                    w_df = self._make_dataframe(r, sort=False, columns=cols)
                     self.db.blocks[v_name] = w_df
                     # self._write_dataframe(w_df, v_name, out_dir)
                 else:
@@ -572,7 +567,7 @@ class ExperimentIndexer(GribIndexer):
 
             # scalar
             LOG.info(f"generate scalar fields index ...")
-            df = self._make_dataframe(data)
+            df = self._make_dataframe(data, sort=True)
             self.db.blocks["scalar"] = df
             self._write_dataframe(df, "scalar", out_dir)
 
@@ -585,7 +580,7 @@ class ExperimentIndexer(GribIndexer):
                     cols = [*self.keys]
                     for i in range(comp_num):
                         cols.extend([f"_msgIndex{i+1}", f"_fileIndex{i+1}"])
-                    w_df = self._make_dataframe(r, columns=cols)
+                    w_df = self._make_dataframe(r, sort=True, columns=cols)
                     # print(f"wind_len={len(w_df.index)}")
                     self.db.blocks[v_name] = w_df
                     self._write_dataframe(w_df, v_name, out_dir)
