@@ -1348,6 +1348,48 @@ def test_fieldset_mean_over_dim_with_empty_fieldset():
         alldata.select(shortName="t", level=850, number=5).mean()
 
 
+def test_fieldset_mean_over_dim_number_with_holes():
+    # compute and check ensemble means when there are holes in the hypercube
+    alldata = mv.read(file_in_testdir("ztu_multi_dim.grib"))
+    a = alldata.select(shortName=["z", "u"])
+    t = alldata.select(shortName="t")
+    t = t.select(level=[850, 500])  # remove some levels from 't'
+    alldata = mv.Fieldset(fields=[a, t])
+
+    num_ens_members = len(mv.unique(alldata.grib_get_long("number")))
+    assert num_ens_members == 6
+    ens_mean = alldata.mean(dim="number")
+    # check general structure of the result
+    assert len(ens_mean) == len(alldata) / num_ens_members
+    assert mv.unique(ens_mean.grib_get_long("level")) == mv.unique(
+        alldata.grib_get_long("level")
+    )
+    assert mv.unique(ens_mean.grib_get_long("step")) == mv.unique(
+        alldata.grib_get_long("step")
+    )
+    assert mv.unique(ens_mean.grib_get_string("shortName")) == mv.unique(
+        alldata.grib_get_string("shortName")
+    )
+    assert len(mv.unique(ens_mean.grib_get_long("number"))) == 1
+    # check values for specific means #1
+    mean1_computed = ens_mean.select(shortName="z", level=1000, step=3)
+    mean1_verified = alldata.select(shortName="z", level=1000, step=3).mean()
+    assert np.array_equal(mean1_computed.values(), mean1_verified.values())
+    assert np.isclose(mean1_computed.values()[0], 1233.7)  # via calculator
+    # check values for specific means #2
+    mean2_computed = ens_mean.select(shortName="t", level=850, step=9)
+    mean2_verified = alldata.select(shortName="t", level=850, step=9).mean()
+    assert np.array_equal(mean2_computed.values(), mean2_verified.values())
+    assert np.isclose(mean2_computed.values()[2], 285.548)  # via calculator
+    # check values for specific means #3
+    mean3_computed = ens_mean.select(shortName="u", level=500, step=6)
+    mean3_verified = alldata.select(shortName="u", level=500, step=6).mean()
+    assert np.array_equal(mean3_computed.values(), mean3_verified.values())
+    # check that missing means are not present
+    mean_missing = ens_mean.select(shortName="t", level=1000)
+    assert len(mean_missing) == 0
+
+
 def test_fieldset_basic_sum():
     alldata = mv.read(file_in_testdir("ztu_multi_dim.grib"))
     m = mv.sum(alldata)  # as function
@@ -2905,3 +2947,41 @@ def test_convolve():
     fieldset_equal(r, f_ref[2:4])
     meta = r.grib_get_long("generatingProcessIdentifier")
     np.testing.assert_allclose(meta, meta_ref)
+
+
+# subprocess.run does not work well with Metview, so we use another method
+# to run a Python process that uses Metview and returns the output
+def _run_external_python_command(cmd, args):
+    import subprocess
+    import sys
+    import tempfile
+
+    fd, path = tempfile.mkstemp()
+
+    run_args = [sys.executable]  # the Python executable
+    run_args.append(cmd)
+    run_args.extend(args)
+
+    # create a clean environment to avoid clashes with the current session
+    env = {"PATH": os.environ["PATH"]}
+    mv_start_cmd = os.environ.get("METVIEW_PYTHON_START_CMD", None)
+    if mv_start_cmd is not None:
+        env.update({"METVIEW_PYTHON_START_CMD": mv_start_cmd})
+    ppath = os.environ.get("PYTHONPATH", None)
+    if ppath is not None:
+        env.update({"PYTHONPATH": ppath})
+
+    with os.fdopen(fd, "w") as f:
+        subprocess.call(run_args, stdout=f, env=env)
+    f = open(path, "r")
+    a = f.read()
+    f.close()
+    return a
+
+
+def test_arguments():
+
+    output = _run_external_python_command(
+        file_in_testdir("args.py"), ["arg1", "arg 2", "59.35", "99.9000"]
+    )
+    assert output == "['arg1', 'arg 2', 59.35, 99.9]\n"
